@@ -35,19 +35,11 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include "gstmad.h"
 #include <gst/audio/audio.h>
 
-
-/* elementfactory information */
-static const GstElementDetails gst_mad_details =
-GST_ELEMENT_DETAILS ("mad mp3 decoder",
-    "Codec/Decoder/Audio",
-    "Uses mad code to decode mp3 streams",
-    "Wim Taymans <wim@fluendo.com>");
-
-
 enum
 {
   ARG_0,
@@ -83,10 +75,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         "channels = (int) [ 1, 2 ]")
     );
 
-static void gst_mad_base_init (gpointer g_class);
-static void gst_mad_class_init (GstMadClass * klass);
-static void gst_mad_init (GstMad * mad, GstMadClass * klass);
 static void gst_mad_dispose (GObject * object);
+static void gst_mad_clear_queues (GstMad * mad);
 
 static void gst_mad_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -105,6 +95,7 @@ static gboolean gst_mad_convert_src (GstPad * pad, GstFormat src_format,
 
 static gboolean gst_mad_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_mad_chain (GstPad * pad, GstBuffer * buffer);
+static GstFlowReturn gst_mad_chain_reverse (GstMad * mad, GstBuffer * buf);
 
 static GstStateChangeReturn gst_mad_change_state (GstElement * element,
     GstStateChange transition);
@@ -114,13 +105,9 @@ static void gst_mad_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mad_get_index (GstElement * element);
 #endif
 
-static void
-_do_init (GType type)
-{
-  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
-}
+static GstTagList *gst_mad_id3_to_tag_list (const struct id3_tag *tag);
 
-GST_BOILERPLATE_FULL (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT, _do_init);
+GST_BOILERPLATE (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT);
 
 /*
 #define GST_TYPE_MAD_LAYER (gst_mad_layer_get_type())
@@ -192,7 +179,9 @@ gst_mad_base_init (gpointer g_class)
       gst_static_pad_template_get (&mad_sink_template_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&mad_src_template_factory));
-  gst_element_class_set_details (element_class, &gst_mad_details);
+  gst_element_class_set_details_simple (element_class, "mad mp3 decoder",
+      "Codec/Decoder/Audio",
+      "Uses mad code to decode mp3 streams", "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -222,10 +211,10 @@ gst_mad_class_init (GstMadClass * klass)
    * default values and ranges need to be selected right */
   g_object_class_install_property (gobject_class, ARG_HALF,
       g_param_spec_boolean ("half", "Half", "Generate PCM at 1/2 sample rate",
-          FALSE, G_PARAM_READWRITE));
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
-      g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
-          TRUE, G_PARAM_READWRITE));
+      g_param_spec_boolean ("ignore-crc", "Ignore CRC", "Ignore CRC errors",
+          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* register tags */
 #define GST_TAG_LAYER    "layer"
@@ -640,6 +629,9 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
+  if (rate < 0.0)
+    return FALSE;
+
   if (format == GST_FORMAT_TIME) {
     gst_segment_set_seek (&mad->segment, rate, format, flags, cur_type,
         cur, stop_type, stop, NULL);
@@ -711,6 +703,9 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
+  if (rate < 0.0)
+    return FALSE;
+
   if (format != GST_FORMAT_TIME) {
     conv = GST_FORMAT_TIME;
     if (!gst_mad_convert_src (pad, format, cur, &conv, &time_cur))
@@ -731,9 +726,6 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   /* shave off the flush flag, we'll need it later */
   flush = ((flags & GST_SEEK_FLAG_FLUSH) != 0);
 
-  /* assume the worst */
-  res = FALSE;
-
   conv = GST_FORMAT_BYTES;
   if (!gst_mad_convert_sink (pad, GST_FORMAT_TIME, time_cur, &conv, &bytes_cur))
     goto convert_error;
@@ -901,7 +893,6 @@ gst_mad_get_property (GObject * object, guint prop_id,
 static void
 gst_mad_update_info (GstMad * mad)
 {
-  gint abr = mad->vbr_average;
   struct mad_header *header = &mad->frame.header;
   gboolean changed = FALSE;
 
@@ -917,7 +908,6 @@ G_STMT_START{                                                   \
   if (mad->new_header) {
     mad->framecount = 1;
     mad->vbr_rate = header->bitrate;
-    abr = 0;
   } else {
     mad->framecount++;
     mad->vbr_rate += header->bitrate;
@@ -1006,6 +996,8 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
+      if (mad->segment.rate < 0.0)
+        gst_mad_chain_reverse (mad, NULL);
       mad->caps_set = FALSE;    /* could be a new stream */
       result = gst_pad_push_event (mad->srcpad, event);
       break;
@@ -1015,9 +1007,10 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       mad->tempsize = 0;
       mad_frame_mute (&mad->frame);
       mad_synth_mute (&mad->synth);
+      gst_mad_clear_queues (mad);
+      /* fall-through */
     case GST_EVENT_FLUSH_START:
       result = gst_pad_event_default (pad, event);
-
       break;
     default:
       if (mad->restart) {
@@ -1098,7 +1091,7 @@ mpg123_parse_xing_header (struct mad_header *header,
   int i;
   guint8 *ptr = (guint8 *) buf;
   double frame_duration;
-  int xflags, xframes, xbytes, xvbr_scale;
+  int xflags, xframes, xbytes;
   int abr;
   guint8 xtoc[XING_TOC_LENGTH];
   int lsf_bit = !(header->flags & MAD_FLAG_LSF_EXT);
@@ -1168,12 +1161,10 @@ mpg123_parse_xing_header (struct mad_header *header,
       ptr += XING_TOC_LENGTH;
     }
 
-    xvbr_scale = -1;
     if (xflags & XING_VBR_SCALE_FLAG) {
       if (ptr >= (buf + bufsize - 4))
         return 0;
-      xvbr_scale = BE_32 (ptr);
-      lprintf ("xvbr_scale: %d\n", xvbr_scale);
+      lprintf ("xvbr_scale: %d\n", BE_32 (ptr));
     }
 
     /* 1 kbit = 1000 bits ! (and not 1024 bits) */
@@ -1281,6 +1272,111 @@ gst_mad_check_caps_reset (GstMad * mad)
   }
 }
 
+static void
+gst_mad_clear_queues (GstMad * mad)
+{
+  g_list_foreach (mad->queued, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (mad->queued);
+  mad->queued = NULL;
+  g_list_foreach (mad->gather, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (mad->gather);
+  mad->gather = NULL;
+  g_list_foreach (mad->decode, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (mad->decode);
+  mad->decode = NULL;
+}
+
+static GstFlowReturn
+gst_mad_flush_decode (GstMad * mad)
+{
+  GstFlowReturn res = GST_FLOW_OK;
+  GList *walk;
+
+  walk = mad->decode;
+
+  GST_DEBUG_OBJECT (mad, "flushing buffers to decoder");
+
+  /* clear buffer and decoder state */
+  mad->tempsize = 0;
+  mad_frame_mute (&mad->frame);
+  mad_synth_mute (&mad->synth);
+
+  mad->process = TRUE;
+  while (walk) {
+    GList *next;
+    GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+
+    GST_DEBUG_OBJECT (mad, "decoding buffer %p, ts %" GST_TIME_FORMAT,
+        buf, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+
+    next = g_list_next (walk);
+    /* decode buffer, resulting data prepended to output queue */
+    gst_buffer_ref (buf);
+    res = gst_mad_chain (mad->sinkpad, buf);
+
+    /* if we generated output, we can discard the buffer, else we
+     * keep it in the queue */
+    if (mad->queued) {
+      GST_DEBUG_OBJECT (mad, "decoded buffer to %p", mad->queued->data);
+      mad->decode = g_list_delete_link (mad->decode, walk);
+      gst_buffer_unref (buf);
+    } else {
+      GST_DEBUG_OBJECT (mad, "buffer did not decode, keeping");
+    }
+    walk = next;
+  }
+  mad->process = FALSE;
+
+  /* now send queued data downstream */
+  while (mad->queued) {
+    GstBuffer *buf = GST_BUFFER_CAST (mad->queued->data);
+
+    GST_DEBUG_OBJECT (mad, "pushing buffer %p of size %u, "
+        "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
+        GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+    res = gst_pad_push (mad->srcpad, buf);
+
+    mad->queued = g_list_delete_link (mad->queued, mad->queued);
+  }
+
+  return res;
+}
+
+static GstFlowReturn
+gst_mad_chain_reverse (GstMad * mad, GstBuffer * buf)
+{
+  GstFlowReturn result = GST_FLOW_OK;
+
+  /* if we have a discont, move buffers to the decode list */
+  if (!buf || GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT)) {
+    GST_DEBUG_OBJECT (mad, "received discont");
+    while (mad->gather) {
+      GstBuffer *gbuf;
+
+      gbuf = GST_BUFFER_CAST (mad->gather->data);
+      /* remove from the gather list */
+      mad->gather = g_list_delete_link (mad->gather, mad->gather);
+      /* copy to decode queue */
+      mad->decode = g_list_prepend (mad->decode, gbuf);
+    }
+    /* decode stuff in the decode queue */
+    gst_mad_flush_decode (mad);
+  }
+
+  if (G_LIKELY (buf)) {
+    GST_DEBUG_OBJECT (mad, "gathering buffer %p of size %u, "
+        "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
+        GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+
+    /* add buffer to gather queue */
+    mad->gather = g_list_prepend (mad->gather, buf);
+  }
+
+  return result;
+}
+
 static GstFlowReturn
 gst_mad_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -1300,11 +1396,19 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     GST_DEBUG ("mad restarted");
   }
 
-  /* take discont flag */
-  discont = GST_BUFFER_IS_DISCONT (buffer);
+  if (mad->segment.rate < 0.0) {
+    if (!mad->process)
+      return gst_mad_chain_reverse (mad, buffer);
+    /* no output discont */
+    discont = FALSE;
+  } else {
+    /* take discont flag */
+    discont = GST_BUFFER_IS_DISCONT (buffer);
+  }
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
-  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
+  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT " duration:%" GST_TIME_FORMAT,
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
 
   /* handle timestamps */
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
@@ -1624,14 +1728,22 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           goto skip_frame;
         }
 
+        if (GST_BUFFER_SIZE (outbuffer) != nsamples * mad->channels * 4) {
+          gst_buffer_unref (outbuffer);
+
+          outbuffer = gst_buffer_new_and_alloc (nsamples * mad->channels * 4);
+          gst_buffer_set_caps (outbuffer, GST_PAD_CAPS (mad->srcpad));
+        }
+
         mad_synth_frame (&mad->synth, &mad->frame);
         left_ch = mad->synth.pcm.samples[0];
         right_ch = mad->synth.pcm.samples[1];
 
         outdata = (gint32 *) GST_BUFFER_DATA (outbuffer);
 
-        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (time_offset));
+        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT " dur: %"
+            GST_TIME_FORMAT, GST_TIME_ARGS (time_offset),
+            GST_TIME_ARGS (time_duration));
 
         GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
         GST_BUFFER_DURATION (outbuffer) = time_duration;
@@ -1668,7 +1780,13 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           }
 
           mad->segment.last_stop = GST_BUFFER_TIMESTAMP (outbuffer);
-          result = gst_pad_push (mad->srcpad, outbuffer);
+          if (mad->segment.rate > 0.0) {
+            result = gst_pad_push (mad->srcpad, outbuffer);
+          } else {
+            GST_LOG_OBJECT (mad, "queued buffer");
+            mad->queued = g_list_prepend (mad->queued, outbuffer);
+            result = GST_FLOW_OK;
+          }
           if (result != GST_FLOW_OK) {
             /* Head for the exit, dropping samples as we go */
             goto_exit = TRUE;
@@ -1787,6 +1905,7 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
         gst_tag_list_free (mad->tags);
         mad->tags = NULL;
       }
+      gst_mad_clear_queues (mad);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -1795,3 +1914,194 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
   }
   return ret;
 }
+
+/* id3 tag helper (FIXME: why does mad parse id3 tags at all? It shouldn't) */
+static GstTagList *
+gst_mad_id3_to_tag_list (const struct id3_tag *tag)
+{
+  const struct id3_frame *frame;
+  const id3_ucs4_t *ucs4;
+  id3_utf8_t *utf8;
+  GstTagList *tag_list;
+  GType tag_type;
+  guint i = 0;
+
+  tag_list = gst_tag_list_new ();
+
+  while ((frame = id3_tag_findframe (tag, NULL, i++)) != NULL) {
+    const union id3_field *field;
+    unsigned int nstrings, j;
+    const gchar *tag_name;
+
+    /* find me the function to query the frame id */
+    gchar *id = g_strndup (frame->id, 5);
+
+    tag_name = gst_tag_from_id3_tag (id);
+    if (tag_name == NULL) {
+      g_free (id);
+      continue;
+    }
+
+    if (strcmp (id, "COMM") == 0) {
+      if (frame->nfields < 4)
+        continue;
+
+      ucs4 = id3_field_getfullstring (&frame->fields[3]);
+      g_assert (ucs4);
+
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
+        continue;
+
+      if (!g_utf8_validate ((char *) utf8, -1, NULL)) {
+        GST_ERROR ("converted string is not valid utf-8");
+        g_free (utf8);
+        continue;
+      }
+
+      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+          GST_TAG_COMMENT, utf8, NULL);
+
+      g_free (utf8);
+      continue;
+    }
+
+    if (frame->nfields < 2)
+      continue;
+
+    field = &frame->fields[1];
+    nstrings = id3_field_getnstrings (field);
+
+    for (j = 0; j < nstrings; ++j) {
+      ucs4 = id3_field_getstrings (field, j);
+      g_assert (ucs4);
+
+      if (strcmp (id, ID3_FRAME_GENRE) == 0)
+        ucs4 = id3_genre_name (ucs4);
+
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
+        continue;
+
+      if (!g_utf8_validate ((char *) utf8, -1, NULL)) {
+        GST_ERROR ("converted string is not valid utf-8");
+        free (utf8);
+        continue;
+      }
+
+      tag_type = gst_tag_get_type (tag_name);
+
+      /* be sure to add non-string tags here */
+      switch (tag_type) {
+        case G_TYPE_UINT:
+        {
+          guint tmp;
+          gchar *check;
+
+          tmp = strtoul ((char *) utf8, &check, 10);
+
+          if (strcmp (tag_name, GST_TAG_DATE) == 0) {
+            GDate *d;
+
+            if (*check != '\0')
+              break;
+            if (tmp == 0)
+              break;
+            d = g_date_new_dmy (1, 1, tmp);
+            tmp = g_date_get_julian (d);
+            g_date_free (d);
+          } else if (strcmp (tag_name, GST_TAG_TRACK_NUMBER) == 0) {
+            if (*check == '/') {
+              guint total;
+
+              check++;
+              total = strtoul (check, &check, 10);
+              if (*check != '\0')
+                break;
+
+              gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                  GST_TAG_TRACK_COUNT, total, NULL);
+            }
+          } else if (strcmp (tag_name, GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
+            if (*check == '/') {
+              guint total;
+
+              check++;
+              total = strtoul (check, &check, 10);
+              if (*check != '\0')
+                break;
+
+              gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                  GST_TAG_ALBUM_VOLUME_COUNT, total, NULL);
+            }
+          }
+
+          if (*check != '\0')
+            break;
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, tmp,
+              NULL);
+          break;
+        }
+        case G_TYPE_UINT64:
+        {
+          guint64 tmp;
+
+          g_assert (strcmp (tag_name, GST_TAG_DURATION) == 0);
+          tmp = strtoul ((char *) utf8, NULL, 10);
+          if (tmp == 0) {
+            break;
+          }
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+              GST_TAG_DURATION, tmp * 1000 * 1000, NULL);
+          break;
+        }
+        case G_TYPE_STRING:{
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+              tag_name, (const gchar *) utf8, NULL);
+          break;
+        }
+          /* handles GST_TYPE_DATE and anything else */
+        default:{
+          GValue src = { 0, };
+          GValue dest = { 0, };
+
+          g_value_init (&src, G_TYPE_STRING);
+          g_value_set_string (&src, (const gchar *) utf8);
+
+          g_value_init (&dest, tag_type);
+          if (g_value_transform (&src, &dest)) {
+            gst_tag_list_add_values (tag_list, GST_TAG_MERGE_APPEND,
+                tag_name, &dest, NULL);
+          } else {
+            GST_WARNING ("Failed to transform tag from string to type '%s'",
+                g_type_name (tag_type));
+          }
+          g_value_unset (&src);
+          g_value_unset (&dest);
+          break;
+        }
+      }
+      free (utf8);
+    }
+    g_free (id);
+  }
+
+  return tag_list;
+}
+
+/* plugin initialisation */
+
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
+
+  return gst_element_register (plugin, "mad", GST_RANK_SECONDARY,
+      gst_mad_get_type ());
+}
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "mad",
+    "mp3 decoding based on the mad library",
+    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
