@@ -45,18 +45,12 @@ typedef gint mpeg2_state_t;
 GST_DEBUG_CATEGORY_STATIC (mpeg2dec_debug);
 #define GST_CAT_DEFAULT (mpeg2dec_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_mpeg2dec_details =
-GST_ELEMENT_DETAILS ("mpeg1 and mpeg2 video decoder",
-    "Codec/Decoder/Video",
-    "Uses libmpeg2 to decode MPEG video streams",
-    "Wim Taymans <wim.taymans@chello.be>");
-
 /* Send a warning message about decoding errors after receiving this many
  * STATE_INVALID return values from mpeg2_parse. -1 means never.
  */
 #define WARN_THRESHOLD (5)
 
+//#define enable_user_data
 #ifdef enable_user_data
 static GstStaticPadTemplate user_data_template_factory =
 GST_STATIC_PAD_TEMPLATE ("user_data",
@@ -78,7 +72,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-yuv, "
-        "format = (fourcc) { YV12, I420, Y42B }, "
+        "format = (fourcc) { YV12, I420, Y42B, Y444 }, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
         "framerate = (fraction) [ 0/1, 2147483647/1 ]")
@@ -91,11 +85,6 @@ static void gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec);
 static void gst_mpeg2dec_finalize (GObject * object);
 static void gst_mpeg2dec_reset (GstMpeg2dec * mpeg2dec);
 
-static void gst_mpeg2dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_mpeg2dec_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 #ifndef GST_DISABLE_INDEX
 static void gst_mpeg2dec_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mpeg2dec_get_index (GstElement * element);
@@ -132,7 +121,7 @@ static const GstEventMask *gst_mpeg2dec_get_event_masks (GstPad * pad);
 
 static GstElementClass *parent_class = NULL;
 
-static gboolean crop_buffer (GstMpeg2dec * mpeg2dec, GstBuffer ** buf);
+static gboolean gst_mpeg2dec_crop_buffer (GstMpeg2dec * dec, GstBuffer ** buf);
 
 /*static guint gst_mpeg2dec_signals[LAST_SIGNAL] = { 0 };*/
 
@@ -178,7 +167,10 @@ gst_mpeg2dec_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&user_data_template_factory));
 #endif
-  gst_element_class_set_details (element_class, &gst_mpeg2dec_details);
+  gst_element_class_set_details_simple (element_class,
+      "mpeg1 and mpeg2 video decoder", "Codec/Decoder/Video",
+      "Uses libmpeg2 to decode MPEG video streams",
+      "Wim Taymans <wim.taymans@chello.be>");
 }
 
 static void
@@ -192,8 +184,6 @@ gst_mpeg2dec_class_init (GstMpeg2decClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->set_property = gst_mpeg2dec_set_property;
-  gobject_class->get_property = gst_mpeg2dec_get_property;
   gobject_class->finalize = gst_mpeg2dec_finalize;
 
   gstelement_class->change_state = gst_mpeg2dec_change_state;
@@ -250,6 +240,12 @@ gst_mpeg2dec_finalize (GObject * object)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (object);
 
+  if (mpeg2dec->index) {
+    gst_object_unref (mpeg2dec->index);
+    mpeg2dec->index = NULL;
+    mpeg2dec->index_id = 0;
+  }
+
   if (mpeg2dec->decoder) {
     GST_DEBUG_OBJECT (mpeg2dec, "closing decoder");
     mpeg2_close (mpeg2dec->decoder);
@@ -265,8 +261,14 @@ gst_mpeg2dec_finalize (GObject * object)
 static void
 gst_mpeg2dec_reset (GstMpeg2dec * mpeg2dec)
 {
+  if (mpeg2dec->index) {
+    gst_object_unref (mpeg2dec->index);
+    mpeg2dec->index = NULL;
+    mpeg2dec->index_id = 0;
+  }
+
   /* reset the initial video state */
-  mpeg2dec->format = MPEG2DEC_FORMAT_NONE;
+  mpeg2dec->format = GST_VIDEO_FORMAT_UNKNOWN;
   mpeg2dec->width = -1;
   mpeg2dec->height = -1;
   gst_segment_init (&mpeg2dec->segment, GST_FORMAT_UNDEFINED);
@@ -295,9 +297,17 @@ gst_mpeg2dec_set_index (GstElement * element, GstIndex * index)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (element);
 
-  mpeg2dec->index = index;
+  GST_OBJECT_LOCK (mpeg2dec);
+  if (mpeg2dec->index)
+    gst_object_unref (mpeg2dec->index);
+  mpeg2dec->index = NULL;
+  mpeg2dec->index_id = 0;
+  if (index) {
+    mpeg2dec->index = gst_object_ref (index);
+    gst_index_get_writer_id (index, GST_OBJECT (element), &mpeg2dec->index_id);
+  }
 
-  gst_index_get_writer_id (index, GST_OBJECT (element), &mpeg2dec->index_id);
+  GST_OBJECT_UNLOCK (mpeg2dec);
 }
 
 static GstIndex *
@@ -305,138 +315,66 @@ gst_mpeg2dec_get_index (GstElement * element)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (element);
 
-  return mpeg2dec->index;
+  return (mpeg2dec->index) ? gst_object_ref (mpeg2dec->index) : NULL;
 }
 #endif
 
-/* see gst-plugins/gst/games/gstvideoimage.c, paint_setup_I420() */
-#define I420_Y_ROWSTRIDE(width) (GST_ROUND_UP_4(width))
-#define I420_U_ROWSTRIDE(width) (GST_ROUND_UP_8(width)/2)
-#define I420_V_ROWSTRIDE(width) ((GST_ROUND_UP_8(I420_Y_ROWSTRIDE(width)))/2)
-
-#define I420_Y_OFFSET(w,h) (0)
-#define I420_U_OFFSET(w,h) (I420_Y_OFFSET(w,h)+(I420_Y_ROWSTRIDE(w)*GST_ROUND_UP_2(h)))
-#define I420_V_OFFSET(w,h) (I420_U_OFFSET(w,h)+(I420_U_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
-
-#define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
-
-static GstBuffer *
-crop_copy_i420_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * input)
+static GstFlowReturn
+gst_mpeg2dec_crop_buffer (GstMpeg2dec * dec, GstBuffer ** buf)
 {
+  GstFlowReturn flow_ret;
+  GstBuffer *inbuf = *buf;
   GstBuffer *outbuf;
-  guint8 *dest, *src;
-  guint outsize, line;
+  guint outsize, c;
 
-  outsize = I420_SIZE (mpeg2dec->width, mpeg2dec->height);
-  GST_LOG_OBJECT (mpeg2dec, "Copying input buffer %ux%u (%u) to output buffer "
-      "%ux%u (%u)", mpeg2dec->decoded_width, mpeg2dec->decoded_height,
-      GST_BUFFER_SIZE (input), mpeg2dec->width, mpeg2dec->height, outsize);
-  outbuf = gst_buffer_new_and_alloc (outsize);
+  outsize = gst_video_format_get_size (dec->format, dec->width, dec->height);
 
-  /* Copy Y first */
-  src = GST_BUFFER_DATA (input);
-  dest = GST_BUFFER_DATA (outbuf);
-  for (line = 0; line < mpeg2dec->height; line++) {
-    memcpy (dest, src, mpeg2dec->width);
-    dest += I420_Y_ROWSTRIDE (mpeg2dec->width);
-    src += I420_Y_ROWSTRIDE (mpeg2dec->decoded_width);
-  }
+  GST_LOG_OBJECT (dec, "Copying input buffer %ux%u (%u) to output buffer "
+      "%ux%u (%u)", dec->decoded_width, dec->decoded_height,
+      GST_BUFFER_SIZE (inbuf), dec->width, dec->height, outsize);
 
-  /* U */
-  src = GST_BUFFER_DATA (input)
-      + I420_U_OFFSET (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
-  dest = GST_BUFFER_DATA (outbuf)
-      + I420_U_OFFSET (mpeg2dec->width, mpeg2dec->height);
-  for (line = 0; line < mpeg2dec->height / 2; line++) {
-    memcpy (dest, src, mpeg2dec->width / 2);
-    dest += I420_U_ROWSTRIDE (mpeg2dec->width);
-    src += I420_U_ROWSTRIDE (mpeg2dec->decoded_width);
-  }
+  flow_ret = gst_pad_alloc_buffer_and_set_caps (dec->srcpad,
+      GST_BUFFER_OFFSET_NONE, outsize, GST_PAD_CAPS (dec->srcpad), &outbuf);
 
-  /* V */
-  src = GST_BUFFER_DATA (input)
-      + I420_V_OFFSET (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
-  dest = GST_BUFFER_DATA (outbuf)
-      + I420_V_OFFSET (mpeg2dec->width, mpeg2dec->height);
-  for (line = 0; line < mpeg2dec->height / 2; line++) {
-    memcpy (dest, src, mpeg2dec->width / 2);
-    dest += I420_V_ROWSTRIDE (mpeg2dec->width);
-    src += I420_V_ROWSTRIDE (mpeg2dec->decoded_width);
-  }
+  if (G_UNLIKELY (flow_ret != GST_FLOW_OK))
+    return flow_ret;
 
-  return outbuf;
-}
+  for (c = 0; c < 3; c++) {
+    const guint8 *src;
+    guint8 *dest;
+    guint stride_in, stride_out;
+    guint c_height, c_width, line;
 
-  /* FIXME: this is unlikely to be right stride-wise and offset-wise */
-static GstBuffer *
-crop_copy_i422_buffer (GstMpeg2dec * mpeg2dec, GstBuffer * input)
-{
-  GstBuffer *outbuf;
-  guint8 *in_data, *out_data;
-  guint line;
+    src =
+        GST_BUFFER_DATA (inbuf) +
+        gst_video_format_get_component_offset (dec->format, c,
+        dec->decoded_width, dec->decoded_height);
+    dest =
+        GST_BUFFER_DATA (outbuf) +
+        gst_video_format_get_component_offset (dec->format, c, dec->width,
+        dec->height);
+    stride_out = gst_video_format_get_row_stride (dec->format, c, dec->width);
+    stride_in =
+        gst_video_format_get_row_stride (dec->format, c, dec->decoded_width);
+    c_height =
+        gst_video_format_get_component_height (dec->format, c, dec->height);
+    c_width = gst_video_format_get_component_width (dec->format, c, dec->width);
 
-  outbuf = gst_buffer_new_and_alloc (mpeg2dec->width * mpeg2dec->height * 2);
-
-  /* Copy Y first */
-  in_data = GST_BUFFER_DATA (input);
-  out_data = GST_BUFFER_DATA (outbuf);
-  for (line = 0; line < mpeg2dec->height; line++) {
-    memcpy (out_data, in_data, mpeg2dec->width);
-    out_data += mpeg2dec->width;
-    in_data += mpeg2dec->decoded_width;
-  }
-
-  /* Now copy U & V */
-  in_data = GST_BUFFER_DATA (input)
-      + mpeg2dec->decoded_width * mpeg2dec->decoded_height;
-  for (line = 0; line < mpeg2dec->height; line++) {
-    memcpy (out_data, in_data, mpeg2dec->width / 2);
-    memcpy (out_data + mpeg2dec->width * mpeg2dec->height / 2,
-        in_data + mpeg2dec->decoded_width * mpeg2dec->decoded_height / 2,
-        mpeg2dec->width / 2);
-    out_data += mpeg2dec->width / 2;
-    in_data += mpeg2dec->decoded_width / 2;
-  }
-
-  return outbuf;
-}
-
-static gboolean
-crop_buffer (GstMpeg2dec * mpeg2dec, GstBuffer ** buf)
-{
-  gboolean result = FALSE;
-  GstBuffer *input = *buf;
-  GstBuffer *outbuf = input;
-
-  /*We crop only if the target region is smaller than the input one */
-  if ((mpeg2dec->decoded_width > mpeg2dec->width) ||
-      (mpeg2dec->decoded_height > mpeg2dec->height)) {
-    /* If we don't know about the format, we just return the original
-     * buffer.
-     */
-    if (mpeg2dec->format == MPEG2DEC_FORMAT_I422 ||
-        mpeg2dec->format == MPEG2DEC_FORMAT_I420 ||
-        mpeg2dec->format == MPEG2DEC_FORMAT_YV12) {
-      /*FIXME:  I have tried to use gst_buffer_copy_on_write, but it
-       *        still have some artifact, so I'me allocating new buffer
-       *        for each frame decoded...
-       */
-      if (mpeg2dec->format == MPEG2DEC_FORMAT_I422) {
-        outbuf = crop_copy_i422_buffer (mpeg2dec, input);
-      } else {
-        outbuf = crop_copy_i420_buffer (mpeg2dec, input);
-      }
-
-      gst_buffer_set_caps (outbuf, GST_PAD_CAPS (mpeg2dec->srcpad));
-      gst_buffer_copy_metadata (outbuf, input, GST_BUFFER_COPY_TIMESTAMPS);
-      gst_buffer_unref (input);
-
-      *buf = outbuf;
-      result = TRUE;
+    for (line = 0; line < c_height; line++) {
+      memcpy (dest, src, c_width);
+      dest += stride_out;
+      src += stride_in;
     }
   }
 
-  return result;
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (dec->srcpad));
+  gst_buffer_copy_metadata (outbuf, inbuf,
+      GST_BUFFER_COPY_TIMESTAMPS | GST_BUFFER_COPY_FLAGS);
+
+  gst_buffer_unref (*buf);
+  *buf = outbuf;
+
+  return GST_FLOW_OK;
 }
 
 static GstFlowReturn
@@ -450,8 +388,9 @@ gst_mpeg2dec_alloc_sized_buf (GstMpeg2dec * mpeg2dec, guint size,
 
     ret = gst_pad_alloc_buffer_and_set_caps (mpeg2dec->srcpad,
         GST_BUFFER_OFFSET_NONE, size, GST_PAD_CAPS (mpeg2dec->srcpad), obuf);
-    if (ret != GST_FLOW_OK)
+    if (ret != GST_FLOW_OK) {
       return ret;
+    }
 
     /* libmpeg2 needs 16 byte aligned buffers... test for this here
      * and if it fails only a single time create our own buffers from
@@ -510,7 +449,7 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec * mpeg2dec, gint64 offset,
   /* ERRORS */
 no_buffer:
   {
-    if (GST_FLOW_IS_FATAL (ret)) {
+    if (ret != GST_FLOW_WRONG_STATE && ret != GST_FLOW_UNEXPECTED) {
       GST_ELEMENT_ERROR (mpeg2dec, RESOURCE, FAILED, (NULL),
           ("Failed to allocate memory for buffer, reason %s",
               gst_flow_get_name (ret)));
@@ -536,38 +475,28 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
 
   if (sequence->width != sequence->chroma_width &&
       sequence->height != sequence->chroma_height) {
-
-    fourcc = GST_STR_FOURCC ("I420");
-    mpeg2dec->format = MPEG2DEC_FORMAT_I420;
-    mpeg2dec->size =
-        I420_SIZE (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
-
-    mpeg2dec->u_offs =
-        I420_U_OFFSET (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
-    mpeg2dec->v_offs =
-        I420_V_OFFSET (mpeg2dec->decoded_width, mpeg2dec->decoded_height);
-
-  } else if (sequence->width == sequence->chroma_width ||
-      sequence->height == sequence->chroma_height) {
-    gint halfsize;
-
-    fourcc = GST_STR_FOURCC ("Y42B");
-    mpeg2dec->format = MPEG2DEC_FORMAT_I422;
-    halfsize = mpeg2dec->decoded_width * mpeg2dec->decoded_height;
-    mpeg2dec->size = halfsize * 2;
-    mpeg2dec->u_offs = halfsize;
-    mpeg2dec->v_offs = halfsize + (halfsize / 2);
+    mpeg2dec->format = GST_VIDEO_FORMAT_I420;
+  } else if ((sequence->width == sequence->chroma_width &&
+          sequence->height != sequence->chroma_height) ||
+      (sequence->width != sequence->chroma_width &&
+          sequence->height == sequence->chroma_height)) {
+    mpeg2dec->format = GST_VIDEO_FORMAT_Y42B;
   } else {
-    g_warning ("mpeg2dec: 4:4:4 format not yet supported");
-    return (FALSE);
+    mpeg2dec->format = GST_VIDEO_FORMAT_Y444;
   }
 
+  fourcc = gst_video_format_to_fourcc (mpeg2dec->format);
+  mpeg2dec->size = gst_video_format_get_size (mpeg2dec->format,
+      mpeg2dec->decoded_width, mpeg2dec->decoded_height);
+  mpeg2dec->u_offs = gst_video_format_get_component_offset (mpeg2dec->format, 1,
+      mpeg2dec->decoded_width, mpeg2dec->decoded_height);
+  mpeg2dec->v_offs = gst_video_format_get_component_offset (mpeg2dec->format, 2,
+      mpeg2dec->decoded_width, mpeg2dec->decoded_height);
+
   if (mpeg2dec->pixel_width == 0 || mpeg2dec->pixel_height == 0) {
-    GValue par = { 0, }
-    , dar = {
-    0,}
-    , dimensions = {
-    0,};
+    GValue par = { 0, };
+    GValue dar = { 0, };
+    GValue dimensions = { 0, };
 
     /* assume display aspect ratio (DAR) of 4:3 */
     g_value_init (&dar, GST_TYPE_FRACTION);
@@ -597,7 +526,8 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
       "height", G_TYPE_INT, mpeg2dec->height,
       "pixel-aspect-ratio", GST_TYPE_FRACTION, mpeg2dec->pixel_width,
       mpeg2dec->pixel_height,
-      "framerate", GST_TYPE_FRACTION, mpeg2dec->fps_n, mpeg2dec->fps_d, NULL);
+      "framerate", GST_TYPE_FRACTION, mpeg2dec->fps_n, mpeg2dec->fps_d,
+      "interlaced", G_TYPE_BOOLEAN, mpeg2dec->interlaced, NULL);
 
   gst_pad_set_caps (mpeg2dec->srcpad, caps);
   gst_caps_unref (caps);
@@ -654,6 +584,9 @@ handle_sequence (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
   mpeg2dec->fps_d = info->sequence->frame_period;
   mpeg2dec->frame_period = info->sequence->frame_period * GST_USECOND / 27;
 
+  mpeg2dec->interlaced =
+      !(info->sequence->flags & SEQ_FLAG_PROGRESSIVE_SEQUENCE);
+
   GST_DEBUG_OBJECT (mpeg2dec,
       "sequence flags: %d, frame period: %d (%g), frame rate: %d/%d",
       info->sequence->flags, info->sequence->frame_period,
@@ -664,6 +597,13 @@ handle_sequence (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
   GST_DEBUG_OBJECT (mpeg2dec, "transfer chars: %d, matrix coef: %d",
       info->sequence->transfer_characteristics,
       info->sequence->matrix_coefficients);
+  GST_DEBUG_OBJECT (mpeg2dec,
+      "FLAGS: CONSTRAINED_PARAMETERS:%d, PROGRESSIVE_SEQUENCE:%d",
+      info->sequence->flags & SEQ_FLAG_CONSTRAINED_PARAMETERS,
+      info->sequence->flags & SEQ_FLAG_PROGRESSIVE_SEQUENCE);
+  GST_DEBUG_OBJECT (mpeg2dec, "FLAGS: LOW_DELAY:%d, COLOUR_DESCRIPTION:%d",
+      info->sequence->flags & SEQ_FLAG_LOW_DELAY,
+      info->sequence->flags & SEQ_FLAG_COLOUR_DESCRIPTION);
 
   if (!gst_mpeg2dec_negotiate_format (mpeg2dec))
     goto negotiate_failed;
@@ -923,11 +863,27 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
   }
   mpeg2dec->next_time += GST_BUFFER_DURATION (outbuf);
 
+  if (picture->flags & PIC_FLAG_TOP_FIELD_FIRST)
+    GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_TFF);
+
+#if MPEG2_RELEASE >= MPEG2_VERSION(0,5,0)
+  /* repeat field introduced in 0.5.0 */
+  if (picture->flags & PIC_FLAG_REPEAT_FIRST_FIELD)
+    GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_RFF);
+#endif
+
   GST_DEBUG_OBJECT (mpeg2dec,
-      "picture: %s %s fields:%d off:%" G_GINT64_FORMAT " ts:%"
+      "picture: %s %s %s %s %s fields:%d off:%" G_GINT64_FORMAT " ts:%"
       GST_TIME_FORMAT,
-      (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff " : "    "),
       (picture->flags & PIC_FLAG_PROGRESSIVE_FRAME ? "prog" : "    "),
+      (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff" : "   "),
+#if MPEG2_RELEASE >= MPEG2_VERSION(0,5,0)
+      (picture->flags & PIC_FLAG_REPEAT_FIRST_FIELD ? "rff" : "   "),
+#else
+      "unknown rff",
+#endif
+      (picture->flags & PIC_FLAG_SKIP ? "skip" : "    "),
+      (picture->flags & PIC_FLAG_COMPOSITE_DISPLAY ? "composite" : "         "),
       picture->nb_fields, GST_BUFFER_OFFSET (outbuf),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
 
@@ -975,8 +931,14 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
    * array of buffers */
   gst_buffer_ref (outbuf);
 
-  /* do cropping if needed */
-  crop_buffer (mpeg2dec, &outbuf);
+  /* do cropping if the target region is smaller than the input one */
+  if (mpeg2dec->decoded_width != mpeg2dec->width ||
+      mpeg2dec->decoded_height != mpeg2dec->height) {
+    GST_DEBUG_OBJECT (mpeg2dec, "cropping buffer");
+    ret = gst_mpeg2dec_crop_buffer (mpeg2dec, &outbuf);
+    if (ret != GST_FLOW_OK)
+      goto done;
+  }
 
   if (mpeg2dec->segment.rate >= 0.0) {
     /* forward: push right away */
@@ -985,6 +947,7 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
         outbuf,
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
+    GST_LOG_OBJECT (mpeg2dec, "... with flags %x", GST_BUFFER_FLAGS (outbuf));
 
     ret = gst_pad_push (mpeg2dec->srcpad, outbuf);
     GST_DEBUG_OBJECT (mpeg2dec, "pushed with result %s",
@@ -997,6 +960,8 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
     ret = GST_FLOW_OK;
   }
 
+done:
+
   return ret;
 
   /* special cases */
@@ -1122,6 +1087,11 @@ gst_mpeg2dec_chain (GstPad * pad, GstBuffer * buf)
     GST_DEBUG_OBJECT (mpeg2dec, "parse state %d", state);
 
     switch (state) {
+#if MPEG2_RELEASE >= MPEG2_VERSION (0, 5, 0)
+      case STATE_SEQUENCE_MODIFIED:
+        GST_DEBUG_OBJECT (mpeg2dec, "sequence modified");
+        /* fall through */
+#endif
       case STATE_SEQUENCE:
         ret = handle_sequence (mpeg2dec, info);
         /* if there is an error handling the sequence
@@ -1348,7 +1318,7 @@ gst_mpeg2dec_sink_convert (GstPad * pad, GstFormat src_format, gint64 src_value,
         case GST_FORMAT_TIME:
           if (info->sequence && info->sequence->byte_rate) {
             *dest_value =
-                gst_util_uint64_scale_int (GST_SECOND, src_value,
+                gst_util_uint64_scale (GST_SECOND, src_value,
                 info->sequence->byte_rate);
             GST_WARNING_OBJECT (mpeg2dec, "dest_value:%" GST_TIME_FORMAT,
                 GST_TIME_ARGS (*dest_value));
@@ -1471,6 +1441,11 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQuery * query)
       GstFormat format;
       gint64 cur;
 
+      /* First, we try to ask upstream, which might know better, especially in
+       * the case of DVDs, with multiple chapter */
+      if ((res = gst_pad_peer_query (mpeg2dec->sinkpad, query)))
+        break;
+
       /* save requested format */
       gst_query_parse_position (query, &format, NULL);
 
@@ -1479,10 +1454,15 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQuery * query)
               mpeg2dec->next_time, &format, &cur))
         goto error;
 
+      cur = gst_segment_to_stream_time (&mpeg2dec->segment, format, cur);
+      if (cur == -1)
+        goto error;
+
       gst_query_set_position (query, format, cur);
 
       GST_LOG_OBJECT (mpeg2dec,
-          "position query: we return %llu (format %u)", cur, format);
+          "position query: we return %" G_GUINT64_FORMAT " (format %u)", cur,
+          format);
       break;
     }
     case GST_QUERY_DURATION:
@@ -1518,8 +1498,8 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQuery * query)
 
       /* get the returned format */
       gst_query_parse_duration (query, &rformat, &total_bytes);
-      GST_LOG_OBJECT (mpeg2dec, "peer pad returned total=%lld bytes",
-          total_bytes);
+      GST_LOG_OBJECT (mpeg2dec,
+          "peer pad returned total=%" G_GINT64_FORMAT " bytes", total_bytes);
 
       if (total_bytes != -1) {
         if (!gst_mpeg2dec_sink_convert (pad, GST_FORMAT_BYTES, total_bytes,
@@ -1532,7 +1512,8 @@ gst_mpeg2dec_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_duration (query, format, total);
 
       GST_LOG_OBJECT (mpeg2dec,
-          "position query: we return %llu (format %u)", total, format);
+          "position query: we return %" G_GUINT64_FORMAT " (format %u)", total,
+          format);
       break;
     }
     default:
@@ -1815,40 +1796,10 @@ init_failed:
   }
 }
 
-static void
-gst_mpeg2dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstMpeg2dec *src;
-
-  g_return_if_fail (GST_IS_MPEG2DEC (object));
-  src = GST_MPEG2DEC (object);
-
-  switch (prop_id) {
-    default:
-      break;
-  }
-}
-
-static void
-gst_mpeg2dec_get_property (GObject * object, guint prop_id, GValue * value,
-    GParamSpec * pspec)
-{
-  GstMpeg2dec *mpeg2dec;
-
-  g_return_if_fail (GST_IS_MPEG2DEC (object));
-  mpeg2dec = GST_MPEG2DEC (object);
-
-  switch (prop_id) {
-    default:
-      break;
-  }
-}
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "mpeg2dec", GST_RANK_SECONDARY,
+  if (!gst_element_register (plugin, "mpeg2dec", GST_RANK_PRIMARY,
           GST_TYPE_MPEG2DEC))
     return FALSE;
 
