@@ -20,28 +20,21 @@
 
 /**
  * SECTION:element-siddec
- * @short_description: a decoder for sid files
  *
- * <refsect2>
- * <para>
  * This element decodes .sid files to raw audio. .sid files are in fact 
  * small Commodore 64 programs that are executed on an emulated 6502 CPU and a 
- * MOS 6581 sound chip. 
- * </para>
- * <para>
+ * MOS 6581 sound chip.
+ * 
  * This plugin will first load the complete program into memory before starting
  * the emulator and producing output.
- * </para>
- * <para>
+ * 
  * Seeking is not (and cannot be) implemented.
- * </para>
+ * 
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=Hawkeye.sid ! siddec ! audioconvert ! alsasink
- * </programlisting>
- * Decode a sid file and play back the audio using alsasink.
- * </para>
+ * ]| Decode a sid file and play back the audio using alsasink.
  * </refsect2>
  *
  * Last reviewed on 2006-12-30 (0.10.5)
@@ -54,19 +47,6 @@
 #include <string.h>
 #include "gstsiddec.h"
 
-static const GstElementDetails gst_siddec_details =
-GST_ELEMENT_DETAILS ("Sid decoder",
-    "Codec/Decoder/Audio",
-    "Use libsidplay to decode SID audio tunes",
-    "Wim Taymans <wim@fluendo.com> ");
-
-/* Sidec signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 #define DEFAULT_TUNE		0
 #define DEFAULT_CLOCK		SIDTUNE_CLOCK_PAL
 #define DEFAULT_MEMORY		MPU_BANK_SWITCHING
@@ -163,8 +143,6 @@ static void gst_siddec_set_property (GObject * object, guint prop_id,
 
 static GstElementClass *parent_class = NULL;
 
-//static guint gst_siddec_signals[LAST_SIGNAL] = { 0 };
-
 GType
 gst_siddec_get_type (void)
 {
@@ -197,7 +175,9 @@ gst_siddec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_siddec_details);
+  gst_element_class_set_details_simple (element_class, "Sid decoder",
+      "Codec/Decoder/Audio", "Use libsidplay to decode SID audio tunes",
+      "Wim Taymans <wim.taymans@gmail.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
@@ -222,49 +202,50 @@ gst_siddec_class_init (GstSidDec * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TUNE,
       g_param_spec_int ("tune", "tune", "tune",
-          0, 100, DEFAULT_TUNE, (GParamFlags) G_PARAM_READWRITE));
+          0, 100, DEFAULT_TUNE,
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CLOCK,
       g_param_spec_enum ("clock", "clock", "clock",
-          GST_TYPE_SID_CLOCK, DEFAULT_CLOCK, (GParamFlags) G_PARAM_READWRITE));
+          GST_TYPE_SID_CLOCK, DEFAULT_CLOCK,
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MEMORY,
       g_param_spec_enum ("memory", "memory", "memory", GST_TYPE_SID_MEMORY,
-          DEFAULT_MEMORY, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_MEMORY,
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FILTER,
       g_param_spec_boolean ("filter", "filter", "filter", DEFAULT_FILTER,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MEASURED_VOLUME,
-      g_param_spec_boolean ("measured_volume", "measured_volume",
+      g_param_spec_boolean ("measured-volume", "measured_volume",
           "measured_volume", DEFAULT_MEASURED_VOLUME,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MOS8580,
       g_param_spec_boolean ("mos8580", "mos8580", "mos8580", DEFAULT_MOS8580,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FORCE_SPEED,
-      g_param_spec_boolean ("force_speed", "force_speed", "force_speed",
-          DEFAULT_FORCE_SPEED, (GParamFlags) G_PARAM_READWRITE));
+      g_param_spec_boolean ("force-speed", "force_speed", "force_speed",
+          DEFAULT_FORCE_SPEED,
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BLOCKSIZE,
       g_param_spec_ulong ("blocksize", "Block size",
           "Size in bytes to output per buffer", 1, G_MAXULONG,
-          DEFAULT_BLOCKSIZE, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_BLOCKSIZE,
+          (GParamFlags)(G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (gobject_class, PROP_METADATA,
       g_param_spec_boxed ("metadata", "Metadata", "Metadata", GST_TYPE_CAPS,
-          (GParamFlags) G_PARAM_READABLE));
+          (GParamFlags)(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
 }
 
 static void
 gst_siddec_init (GstSidDec * siddec)
 {
-  siddec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_templ),
-      "sink");
+  siddec->sinkpad = gst_pad_new_from_static_template (&sink_templ, "sink");
   gst_pad_set_query_function (siddec->sinkpad, NULL);
   gst_pad_set_event_function (siddec->sinkpad, gst_siddec_sink_event);
   gst_pad_set_chain_function (siddec->sinkpad, gst_siddec_chain);
   gst_element_add_pad (GST_ELEMENT (siddec), siddec->sinkpad);
 
-  siddec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_templ),
-      "src");
+  siddec->srcpad = gst_pad_new_from_static_template (&src_templ, "src");
   gst_pad_set_event_function (siddec->srcpad, gst_siddec_src_event);
   gst_pad_set_query_function (siddec->srcpad, gst_siddec_src_query);
   gst_pad_use_fixed_caps (siddec->srcpad);
@@ -316,15 +297,15 @@ update_tags (GstSidDec * siddec)
 
     if (info.nameString) {
       gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-          GST_TAG_TITLE, info.nameString, NULL);
+          GST_TAG_TITLE, info.nameString, (void *) NULL);
     }
     if (info.authorString) {
       gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-          GST_TAG_ARTIST, info.authorString, NULL);
+          GST_TAG_ARTIST, info.authorString, (void *) NULL);
     }
     if (info.copyrightString) {
       gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-          GST_TAG_COPYRIGHT, info.copyrightString, NULL);
+          GST_TAG_COPYRIGHT, info.copyrightString, (void *) NULL);
     }
     gst_element_found_tags_for_pad (GST_ELEMENT_CAST (siddec),
         siddec->srcpad, list);
@@ -454,20 +435,18 @@ pause:
   {
     const gchar *reason = gst_flow_get_name (ret);
 
-    GST_DEBUG_OBJECT (siddec, "pausing task, reason %s", reason);
-    gst_pad_pause_task (pad);
-
-    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-      if (ret == GST_FLOW_UNEXPECTED) {
-        /* perform EOS logic, FIXME, segment seek? */
-        gst_pad_push_event (pad, gst_event_new_eos ());
-      } else {
-        /* for fatal errors we post an error message */
-        GST_ELEMENT_ERROR (siddec, STREAM, FAILED,
-            (NULL), ("streaming task paused, reason %s", reason));
-        gst_pad_push_event (pad, gst_event_new_eos ());
-      }
+    if (ret == GST_FLOW_UNEXPECTED) {
+      /* perform EOS logic, FIXME, segment seek? */
+      gst_pad_push_event (pad, gst_event_new_eos ());
+    } else  if (ret < GST_FLOW_UNEXPECTED || ret == GST_FLOW_NOT_LINKED) {
+      /* for fatal errors we post an error message */
+      GST_ELEMENT_ERROR (siddec, STREAM, FAILED,
+          (NULL), ("streaming task paused, reason %s", reason));
+      gst_pad_push_event (pad, gst_event_new_eos ());
     }
+
+    GST_INFO_OBJECT (siddec, "pausing task, reason: %s", reason);
+    gst_pad_pause_task (pad);
     goto done;
   }
 }
@@ -793,8 +772,9 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_SIDDEC);
 }
 
+/* FIXME: remove cast to gchar once we depend on core >= 0.10.14.1 */
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "siddec",
-    "Uses libsidplay to decode .sid files",
+    (gchar *) "Uses libsidplay to decode .sid files",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
