@@ -52,7 +52,6 @@ struct _GstX264Enc
 
   x264_t *x264enc;
   x264_param_t x264param;
-  gint current_byte_stream;
 
   /* properties */
   guint threads;
@@ -113,14 +112,6 @@ struct _GstX264Enc
 
   gint i_type;
   GstEvent *forcekeyunit_event;
-
-  /* configuration changed  while playing */
-  gboolean reconfig;
-
-  /* from the downstream caps */
-  const gchar *peer_profile;
-  gboolean peer_intra_profile;
-  const x264_level_t *peer_level;
 };
 
 struct _GstX264EncClass
