@@ -321,10 +321,10 @@ gst_mp3parse_class_init (GstMPEGAudioParseClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SKIP,
       g_param_spec_int ("skip", "skip", "skip",
-          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BIT_RATE,
       g_param_spec_int ("bitrate", "Bitrate", "Bit Rate",
-          G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_mp3parse_change_state;
 
@@ -1527,7 +1527,7 @@ gst_mp3parse_handle_data (GstMPEGAudioParse * mp3parse, gboolean at_eos)
     if (!mp3parse->skip) {
       mp3parse->resyncing = FALSE;
       flow = gst_mp3parse_emit_frame (mp3parse, bpf, mode, crc);
-      if (GST_FLOW_IS_FATAL (flow))
+      if (flow != GST_FLOW_OK)
         break;
     } else {
       GST_DEBUG_OBJECT (mp3parse, "skipping buffer of %d bytes", bpf);
