@@ -41,15 +41,6 @@ GST_DEBUG_CATEGORY_STATIC (gstdvddemux_debug);
 #define DEMUX_CLASS(o)  GST_MPEG_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 #define CLASS(o)  GST_DVD_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 
-
-/* Element factory information */
-static GstElementDetails dvd_demux_details = {
-  "DVD Demuxer",
-  "Codec/Demuxer",
-  "Demultiplexes DVD (VOB) MPEG2 streams",
-  "Martin Soto <martinsoto@users.sourceforge.net>"
-};
-
 /* DVDDemux signals and args */
 enum
 {
@@ -134,11 +125,6 @@ GST_STATIC_PAD_TEMPLATE ("current_subpicture",
 GST_BOILERPLATE_FULL (GstDVDDemux, gst_dvd_demux, GstMPEGDemux,
     GST_TYPE_MPEG_DEMUX, _do_init);
 
-static void gst_dvd_demux_class_init (GstDVDDemuxClass * klass);
-static void gst_dvd_demux_base_init (gpointer klass);
-static void gst_dvd_demux_init (GstDVDDemux * dvd_demux,
-    GstDVDDemuxClass * klass);
-
 static gboolean gst_dvd_demux_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 static gboolean gst_dvd_demux_parse_packhead (GstMPEGParse * mpeg_parse,
@@ -227,7 +213,10 @@ gst_dvd_demux_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       dvd_demux_class->cur_subpicture_template);
 
-  gst_element_class_set_details (element_class, &dvd_demux_details);
+  gst_element_class_set_details_simple (element_class, "DVD Demuxer",
+      "Codec/Demuxer",
+      "Demultiplexes DVD (VOB) MPEG2 streams",
+      "Martin Soto <martinsoto@users.sourceforge.net>");
 }
 
 
@@ -974,7 +963,7 @@ gst_dvd_demux_combine_flows (GstMPEGDemux * mpegdemux, GstMPEGStream * stream,
   stream->last_flow = flow;
 
   /* if it's success we can return the value right away */
-  if (GST_FLOW_IS_SUCCESS (flow))
+  if (flow == GST_FLOW_OK)
     goto done;
 
   /* any other error that is not-linked can be returned right
